<?php
/* vim: set tabstop=4 shiftwidth=4: */

/*
 * Functor Utilities
 * (based on org.apache.commons.collections.functors.FunctorUtils)
 * 
 *
 * PHP version 5
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * 	http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category   PHP
 * @package    Commons
 * @author     Yomei Komiya
 * @copyright  2007 the original author or authors.
 * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
 * @version    SVN: $Id: FunctorUtils.php 105 2008-01-05 12:17:24Z whitestar $
 * @link       http://phpcommons.sourceforge.jp/
 * @see        
 * @since      File available since Release 0.8.0
 */

//namespace Commons::Cols::Functors;

require_once 'Commons/Lang/Object.php';
require_once 'Commons/Lang/IllegalArgumentException.php';
require_once 'Commons/Cols/Closure.php';
require_once 'Commons/IO/Serializable.php';
/*
use Commons::Lang::Object;
use Commons::Lang::IllegalArgumentException;
use Commons::Cols::Closure;
use Commons::IO::Serializable;
*/

/**
 * FunctorUtils
 *
 *
 * @category   PHP
 * @package    Commons.Cols.Functors
 * @author     Yomei Komiya
 * @copyright  2007 the original author or authors.
 * @license    http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
 * @version    Release: 1.0.0
 * @link       http://phpcommons.sourceforge.jp/
 * @see        
 * @since      Class available since Release 0.8.0
 */
abstract class Commons_Cols_Functors_FunctorUtils extends Commons_Lang_Object {
	
	/**
	 * Copies functors.
	 *
	 * @param array $functors
	 * @return array cloned functors.
	 */
	public static function copy($functors) {
		if (is_null($functors)) {
			return null;
		}
		$clonedFunctors = $functors;
		return $clonedFunctors;
	}


	/**
	 * Validates functors.
	 *
	 * @param array $functors
	 * @throws Commons_Lang_IllegalArgumentException if the passed functors is invalid.
	 */
	public static function validate($functors) {
		if (is_null($functors)) {
			throw new Commons_Lang_IllegalArgumentException(
					'The closure array must not be null');
		}
		$i = 0;
		foreach ($functors as $functor) {
			if (is_null($functor)) {
				throw new Commons_Lang_IllegalArgumentException(
						'The closure array must not contain a null closure, index '
						. $i . ' was null');
			}
			$i++;
		}
	}
	
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * c-hanging-comment-ender-p: nil
 * End:
 */
?>